import 'package:amap_flutter_base/amap_flutter_base.dart';
import 'package:amap_flutter_map/src/core/method_channel_amap_flutter_map.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/services.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import '../types/types.dart';
import 'map_event.dart';

/// “amap_flutter_map”平台特定实现必须扩展的接口
abstract class AMapFlutterPlatform extends PlatformInterface {
  static final Object _token = Object();

  AMapFlutterPlatform() : super(token: _token);
  static AMapFlutterPlatform _instance = MethodChannelAMapFlutterMap();

  /// The default instance of [AMapFlutterPlatform] to use.
  ///
  /// Defaults to [MethodChannelAMapFlutterMap].
  static AMapFlutterPlatform get instance => _instance;

  static set instance(AMapFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// /// Initializes the platform interface with [id].
  ///
  /// This method is called when the plugin is first initialized.
  Future<void> init(int id) {
    throw UnimplementedError('init() has not been implemented.');
  }

  void dispose({required int id}) {
    throw UnimplementedError('dispose() has not been implemented.');
  }

  Widget buildView(
      Map<String, dynamic> creationParams,
      Set<Factory<OneSequenceGestureRecognizer>> gestureRecognizers,
      PlatformViewCreatedCallback onPlatformViewCreated) {
    throw UnimplementedError('buildView() has not been implemented.');
  }

  /// 定位回调
  Stream<LocationChangedEvent> onLocationChanged({required int mapId}) {
    throw UnimplementedError('onLocationChanged() has not been implemented.');
  }

  /// Camera 移动回调
  Stream<CameraPositionMoveEvent> onCameraMove({required int mapId}) {
    throw UnimplementedError('onCameraMove() has not been implemented.');
  }

  /// Camera 移动结束回调
  Stream<CameraPositionMoveEndEvent> onCameraMoveEnd({required int mapId}) {
    throw UnimplementedError('onCameraMoveEnd() has not been implemented.');
  }

  ///Camera 搜索POI回调
  Stream<MapSearchPOIEvent> mapSearchPOI({required int mapId}) {
    throw UnimplementedError('mapSearchPOI() has not been implemented.');
  }

  ///Camera 搜索inputtips回调
  Stream<MapSearchInputTipsEvent> mapSearchInputTips({required int mapId}) {
    throw UnimplementedError('mapSearchInputTips() has not been implemented.');
  }

  ///Camera 搜索regeocode回调
  Stream<MapSearchRegeocodeEvent> mapSearchRegeocode({required int mapId}) {
    throw UnimplementedError('mapSearchRegeocode() has not been implemented.');
  }

  /// 地图点击回调
  Stream<MapTapEvent> onMapTap({required int mapId}) {
    throw UnimplementedError('onMapTap() has not been implemented.');
  }

  /// 地图长按回调
  Stream<MapLongPressEvent> onMapLongPress({required int mapId}) {
    throw UnimplementedError('onMapLongPress() has not been implemented.');
  }

  /// poi点击回调
  Stream<MapPoiTouchEvent> onPoiTouched({required int mapId}) {
    throw UnimplementedError('onPoiTouched() has not been implemented.');
  }

  /// marker点击回调
  Stream<MarkerTapEvent> onMarkerTap({required int mapId}) {
    throw UnimplementedError('onMarkerTap() has not been implemented.');
  }

  /// marker拖拽回调
  Stream<MarkerDragEndEvent> onMarkerDragEnd({required int mapId}) {
    throw UnimplementedError('onMarkerDragEnd() has not been implemented.');
  }

  /// polyline点击回调
  Stream<PolylineTapEvent> onPolylineTap({required int mapId}) {
    throw UnimplementedError('onPolylineTap() has not been implemented.');
  }

  /// 更新地图参数
  Future<void> updateMapOptions(
    Map<String, dynamic> newOptions, {
    required int mapId,
  }) {
    throw UnimplementedError('updateMapOptions() has not been implemented.');
  }

  /// 更新Marker的数据
  Future<void> updateMarkers(MarkerUpdates markerUpdates,
      {required int mapId}) {
    throw UnimplementedError('updateMarkers() has not been implemented.');
  }

  /// 更新polyline的数据
  Future<void> updatePolylines(PolylineUpdates polylineUpdates,
      {required int mapId}) {
    throw UnimplementedError('updatePolylines() has not been implemented.');
  }

  /// 更新polygon的数据
  Future<void> updatePolygons(PolygonUpdates polygonUpdates,
      {required int mapId}) {
    throw UnimplementedError('updatePolygons() has not been implemented.');
  }

  /// 移动镜头到一个新的位置
  Future<void> moveCamera(
    CameraUpdate cameraUpdate, {
    required int mapId,
    bool animated = true,
    int duration = 0,
  }) {
    throw UnimplementedError('moveCamera() has not been implemented.');
  }

  /// 展示marker InfoWindow
  Future<void> showInfoWindow(String markerId, {required int mapId}) {
    throw UnimplementedError('showInfoWindow() has not been implemented.');
  }

  /// 隐藏marker InfoWindow
  Future<void> hideInfoWindow({required int mapId}) {
    throw UnimplementedError('hideInfoWindow() has not been implemented.');
  }

  /// 设置地图每秒渲染的帧数
  Future<void> setRenderFps(int fps, {required int mapId}) {
    throw UnimplementedError('setRenderFps() has not been implemented.');
  }

  /// 截屏
  Future<Uint8List?> takeSnapshot({required int mapId}) {
    throw UnimplementedError('takeSnapshot() has not been implemented.');
  }

  ///判断 点坐标是否在polygon 多边形内
  ///@param point 点坐标
  Future<bool?> containsPoint(LatLng point, String id, {required int mapId}) {
    throw UnimplementedError('takeSnapshot() has not been implemented.');
  }

  /// 地图可视区域
  Future<dynamic> getVisibleRegion({required int mapId}) {
    throw UnimplementedError('getVisibleRegion() has not been implemented.');
  }

  /// 地图可视区域
  Future<void> searchPoi(String query, int searchType,
      {required int mapId, double latitude = 0.0, double longitude = 0.0}) {
    throw UnimplementedError('searchPoi() has not been implemented.');
  }

  /// 地图搜索inputTip
  Future<void> searchInput(String query, {required int mapId}) {
    throw UnimplementedError('searchInput() has not been implemented.');
  }

  /// 地图搜索regeocode
  Future<void> searchRegeocode(double latitude, double longitude,
      {required int mapId}) {
    throw UnimplementedError('searchRegeocode() has not been implemented.');
  }

  /// 获取地图审图号（普通地图）
  Future<String?> getMapContentApprovalNumber({required int mapId}) {
    throw UnimplementedError(
        'getMapContentApprovalNumber() has not been implemented.');
  }

  /// 获取地图审图号（卫星地图）
  Future<String?> getSatelliteImageApprovalNumber({required int mapId}) {
    throw UnimplementedError(
        'getSatelliteImageApprovalNumber() has not been implemented.');
  }

  /// 清空缓存
  Future<void> clearDisk({required int mapId}) {
    throw UnimplementedError('clearDisk() has not been implemented.');
  }

  ///只用于测试
  ///用于与native的通信
  MethodChannel channel(int mapId) {
    throw UnimplementedError('channel() has not been implemented.');

  }
}
